      subroutine read_control
C
C Read molecular data
C
      include 'DIMENSIONS'
#ifdef MP
      include 'mpif.h'
      logical OKRandom, prng_restart
      real*8  r1
#endif
      include 'COMMON.IOUNITS'
      include 'COMMON.TIME1'
      include 'COMMON.THREAD'
      include 'COMMON.SBRIDGE'
      include 'COMMON.CONTROL'
      include 'COMMON.MCM'
      include 'COMMON.MAP'
      include 'COMMON.HEADER'
      include 'COMMON.CSA'
      include 'COMMON.ZSCORE'
      include 'COMMON.RMSDBANK'
      COMMON /MACHSW/ KDIAG,ICORFL,IXDR
      character*8 diagmeth(0:3) /'Library','EVVRSP','Givens','Jacobi'/
      character*80 ucase
      character*320 controlcard
#ifdef MPI
      include 'COMMON.INFO'
      integer iseed_array(4)
#endif

      nglob_csa=0
      eglob_csa=1d99
      nmin_csa=0
      read (INP,'(a)') titel
      call card_concat(controlcard)

#ifdef MPL 
C Only the master will set random seed.
      if (MyID.eq.MasterId) then
#endif
        call reada(controlcard,'SEED',seed,0.0D0)
        iseed=-dint(dabs(seed))
#ifndef MPL
        write (iout,'(a,f20.0,i20)') 'Random seed:',seed,iseed
        write (*,'(a,f20.0,i20)') 'Random seed:',seed,iseed
#endif
        if (iseed.eq.0) then
          write (iout,'(/80(1h*)/20x,a/80(1h*))') 
     &     'Random seed undefined. The program will stop.'
          write (*,'(/80(1h*)/20x,a/80(1h*))') 
     &     'Random seed undefined. The program will stop.'
#ifdef MPL
          call mp_stopall(error_msg)
#endif
#ifdef MPI
          call mpi_abort(mpi_comm_world,error_msg,ierr)
#else
          stop 'Bad random seed.'
#endif
        endif
#ifdef MPI
      seed=seed*(MyID+1)+1
      do i=1,4
       tmp=65536.0d0**(4-i)
       iseed_array(i) = dint(seed/tmp)
       seed=seed-iseed_array(i)*tmp
      enddo
      write (iout,*) 'MPI: node= ', MyID, ' iseed(4)= ',
     &                 (iseed_array(i),i=1,4)
      write (*,*) 'MPI: node= ',MyID, ' iseed(4)= ',
     &                 (iseed_array(i),i=1,4)
      OKRandom = prng_restart(MyID,iseed_array)
      if (OKRandom) then
c        r1 = prng_next(MyID)
         r1=ran_number(0.0D0,1.0D0)
        write (iout,*) 'ran_num',r1
        if (r1.lt.0.0d0) OKRandom=.false.
      endif
      if (.not.OKRandom) then
        write (iout,*) 'PRNG IS NOT WORKING!!!'
        write (*,*)    'PRNG IS NOT WORKING!!!'
        if (MyID.eq.0) then 
         call mpi_abort(mpi_comm_world,error_msg,ierr)
         stop
        else
         write (iout,*) 'too many processors for parallel prng'
         write (*,*) 'too many processors for parallel prng'
        endif
      endif
#else
        call vrndst(iseed)
        write (iout,*) 'ran_num',ran_number(0.0d0,1.0d0)
#ifdef MPL
        write (iout,'(a,i10)') 'Random seed:',iseed
        do iproc=fgProcs,nprocs-1,fgProcs
          seed=ran_number(1.0D2,1.0D7)
          iseed=-dabs(seed)
          call mp_send(iseed,4,iproc,SeedType,SeedId) 
        enddo   
      else if (MyID.eq.BossID) then
        call mp_brecv(iseed,4,MasterId,SeedType,SeedId)
        call vrndst(iseed)
        write (iout,'(a,i10)') 'Random seed:',iseed
      endif
#endif
#endif

C Set up the time limit (caution! The time must be input in minutes!)
      call reada(controlcard,'TIMLIM',timlim,60.0D0) 
      unres_pdb = index(controlcard,'UNRES_PDB') .gt. 0
      call reada(controlcard,'SAFETY',safety,15.0D0)
      call reada(controlcard,'RMSDBC',rmsdbc,3.0D0)
      call reada(controlcard,'RMSDBC1',rmsdbc1,0.5D0)
      call reada(controlcard,'RMSDBC1MAX',rmsdbc1max,1.5D0)
      call reada(controlcard,'RMSDBCM',rmsdbcm,3.0D0)
      call reada(controlcard,'DRMS',drms,0.1D0)
      write (iout,'(a,f10.1)')'RMSDBC = ',rmsdbc 
      write (iout,'(a,f10.1)')'RMSDBC1 = ',rmsdbc1 
      write (iout,'(a,f10.1)')'RMSDBC1MAX = ',rmsdbc1max 
      write (iout,'(a,f10.1)')'DRMS    = ',drms 
      write (iout,'(a,f10.1)')'RMSDBCM = ',rmsdbcm 
      write (iout,'(a,f10.1)') 'Time limit (min):',timlim
      call readi(controlcard,'NZ_START',nz_start,0)
      call readi(controlcard,'NZ_END',nz_end,0)
      call readi(controlcard,'IZ_SC',iz_sc,0)
      timlim=60.0D0*timlim
      safety = 60.0d0*safety
      timem=timlim
        
      modecalc=0
      minim=(index(controlcard,'MINIMIZE').gt.0)
      overlapsc=(index(controlcard,'OVERLAP').gt.0)
      sideadd=(index(controlcard,'SIDEADD').gt.0)
      energy_dec=(index(controlcard,'ENERGY_DEC').gt.0)
      outpdb=(index(controlcard,'PDBOUT').gt.0)
      outmol2=(index(controlcard,'MOL2OUT').gt.0)
      pdbref=(index(controlcard,'PDBREF').gt.0)
      refstr=pdbref .or. (index(controlcard,'REFSTR').gt.0)
      indpdb=index(controlcard,'PDBSTART')
      call readi(controlcard,'IPRINT',iprint,0)
      call readi(controlcard,'MAXGEN',maxgen,10000)
      call readi(controlcard,'MAXOVERLAP',maxoverlap,1000)
      call readi(controlcard,"KDIAG",kdiag,0)
      if (index(controlcard,'REGULAR').gt.0.0D0) then
        call reada(controlcard,'WEIDIS',weidis,0.1D0)
        modecalc=1
        refstr=.true.
      endif
      if (index(controlcard,'CHECKGRAD').gt.0) then
        modecalc=5
        if (index(controlcard,'CART').gt.0) then
          icheckgrad=1
        elseif (index(controlcard,'CARINT').gt.0) then
          icheckgrad=2
        else
          icheckgrad=3
        endif
      elseif (index(controlcard,'THREAD').gt.0) then
        modecalc=2
        call readi(controlcard,'THREAD',nthread,0)
        if (nthread.gt.0) then
          call reada(controlcard,'WEIDIS',weidis,0.1D0)
        else
          write (iout,'(a)')'A number has to follow the THREAD keyword.'
          stop 'Error termination in Read_Control.'
        endif
      else if (index(controlcard,'MCMA').gt.0) then
        modecalc=3
      else if (index(controlcard,'MCEE').gt.0) then
        modecalc=6
      else if (index(controlcard,'MULTCONF').gt.0) then
        modecalc=4
      else if (index(controlcard,'MAP').gt.0) then
        modecalc=7
        call readi(controlcard,'MAP',nmap,0)
      else if (index(controlcard,'CSA').gt.0) then
        modecalc=8
      else if (index(controlcard,'ZSCORE').gt.0) then
        modecalc=9
      else if (index(controlcard,'MCMF').gt.0) then
        modecalc=10
      else if (index(controlcard,'SOFTREG').gt.0) then
        modecalc=11
      else if (index(controlcard,'TEST').gt.0) then
        modecalc=-2
      else if (index(controlcard,'MD').gt.0) then
        modecalc=12	
      endif

      iscode=index(controlcard,'ONE_LETTER')
      indphi=index(controlcard,'PHI')
      indback=index(controlcard,'BACK')
      iranconf=index(controlcard,'RAND_CONF')
	i2ndstr=index(controlcard,'USE_SEC_PRED')
cd    write (iout,'(a)') controlcard
cd    write (iout,*) 'IRANCONF',IRANCONF
#ifdef MPL
cd    write (iout,*) 'Processor:',MyId,' ISEED,TIMLIM,MODECALC,ISCODE',
cd   &    iseed,timlim,modecalc,iscode
cd    write (*,*) 'Processor:',MyId,' ISEED,TIMLIM,MODECALC,ISCODE',
cd   &    iseed,timlim,modecalc,iscode
cd    write (iout,*) 'IPDB,INDPHI,INDBACK,IRANCONF,NTHREAD,WEIDIS',
cd   &    indpdb,indphi,indback,iranconf,nthread,weidis
cd    write (*,*) 'IPDB,INDPHI,INDBACK,IRANCONF,NTHREAD,WEIDIS',
cd   &    indpdb,indphi,indback,iranconf,nthread,weidis
#endif
      write (iout,'(2a)') diagmeth(kdiag),
     &  ' routine used to diagonalize matrices.'
      return
      end
c--------------------------------------------------------------------------
      subroutine read_MDpar
C
C Read MD settings
C
      include 'DIMENSIONS'
      include 'COMMON.IOUNITS'
      include 'COMMON.TIME1'
      include 'COMMON.MD'
      include 'COMMON.LANGEVIN'
      include 'COMMON.INTERACT'
      include 'COMMON.NAMES'
      include 'COMMON.GEO'
      character*80 ucase
      character*320 controlcard

      call card_concat(controlcard)
      call readi(controlcard,"NSTEP",n_timestep,1000000)
      call readi(controlcard,"NTWE",ntwe,100)
      call readi(controlcard,"NTWX",ntwx,1000)
      call reada(controlcard,"DT",d_time,1.0d-1)
      call reada(controlcard,"DVMAX",dvmax,2.0d1)
      call reada(controlcard,"DAMAX",damax,1.0d0)
      call reada(controlcard,"EDRIFTMAX",edriftmax,1.0d+1)
      call readi(controlcard,"LANG",lang,0)
      RESPA = index(controlcard,"RESPA") .gt. 0
      call readi(controlcard,"NTIME_SPLIT",ntime_split,10)
      rest = index(controlcard,"REST").gt.0
      tbf = index(controlcard,"TBF").gt.0
      call reada(controlcard,"T_BATH",t_bath,300.0d0)
      call reada(controlcard,"TAU_BATH",tau_bath,1.0d-1) 
      call readi(controlcard,"RESET_MOMENT",count_reset_moment,1000)
      call readi(controlcard,"RESET_VEL",count_reset_vel,1000)
      reset_moment=lang.eq.0 .and. tbf .and. count_reset_moment.gt.0
      reset_vel=lang.eq.0 .and. tbf .and. count_reset_vel.gt.0
      if (count_reset_vel.eq.0) count_reset_vel=1000000000
      large = index(controlcard,"LARGE").gt.0
      print_compon = index(controlcard,"PRINT_COMPON").gt.0
      rattle = index(controlcard,"RATTLE").gt.0
c
      write (iout,*)
      write (iout,'(27(1h=),a26,27(1h=))') " Parameters of the MD run "
      write (iout,*)
      write (iout,'(a)') "The units are:"
      write (iout,'(a)') "positions: angstrom, time: 48.9 fs"
      write (iout,'(2a)') "velocity: angstrom/(48.9 fs),",
     & " acceleration: angstrom/(48.9 fs)**2"
      write (iout,'(a)') "energy: kcal/mol, temperature: K"
      write (iout,*)
      write (iout,'(a60,i10)') "Number of time steps:",n_timestep
      write (iout,'(a60,f10.5,a)') 
     & "Initial time step of numerical integration:",d_time,
     & " natural units"
      write (iout,'(60x,f10.5,a)') d_time*48.9," fs"
      if (RESPA) then
        write (iout,'(2a,i4,a)') 
     &    "RESPA splitting algorithm used; time step for fast-varying",
     &    " short-range forces split into",ntime_split," steps."
      else
        write (iout,'(a60,f10.5)') 
     & "Maximum acceleration threshold to scale down the time step:",
     & damax
        write (iout,'(a60,f10.5)') 
     & "Maximum predicted energy drift to reduce the timestep:",
     &   edriftmax
      endif
      write (iout,'(a60,f10.5)') 
     & "Maximum velocity threshold to reduce velocities:",dvmax
      write (iout,'(a60,i10)') "Frequency of property output:",ntwe
      write (iout,'(a60,i10)') "Frequency of coordinate output:",ntwx
      if (rattle) write (iout,'(a60)') 
     & "Rattle algorithm used to constrain the virtual bonds"
c
      reset_fricmat=1000
      if (lang.gt.0) then
        call reada(controlcard,"ETAWAT",etawat,0.8904d0)
        call reada(controlcard,"RWAT",rwat,1.4d0)
        call reada(controlcard,"SCAL_FRIC",scal_fric,2.0d-2)
        surfarea=index(controlcard,"SURFAREA").gt.0
        call readi(controlcard,"RESET_FRICMAT",reset_fricmat,1000)
        write (iout,'(/a,$)') "Langevin dynamics calculation"
        if (lang.eq.1) then
          write (iout,'(a/)') 
     &      " with direct integration of Langevin equations"  
        else if (lang.eq.2) then
          write (iout,'(a/)') " with TINKER stochasic MD integrator"
        else if (lang.eq.3) then
          write (iout,'(a/)') " with Ciccotti's stochasic MD integrator"
        else if (lang.eq.4) then
          write (iout,'(a/)') " in overdamped mode"
        else
          write (iout,'(//a,i5)') 
     &      "=========== ERROR: Unknown Langevin dynamics mode:",lang
          stop
        endif
        write (iout,'(a60,f10.5)') "Temperature:",t_bath
        write (iout,'(a60,f10.5)') "Viscosity of the solvent:",etawat
        write (iout,'(a60,f10.5)') "Radius of solvent molecule:",rwat
        write (iout,'(a60,f10.5)') 
     &   "Scaling factor of the friction forces:",scal_fric
        if (surfarea) write (iout,'(2a,i10,a)') 
     &     "Friction coefficients will be scaled by solvent-accessible",
     &     " surface area every",reset_fricmat," steps."
c Calculate friction coefficients and bounds of stochastic forces
        eta=6*pi*cPoise*etawat
        write(iout,'(a60,f10.5)')"Eta of the solvent in natural units:",
     &   eta
        gamp=scal_fric*(pstok+rwat)*eta
        stdfp=dsqrt(2*Rb*t_bath/d_time)
        do i=1,ntyp
          gamsc(i)=scal_fric*(restok(i)+rwat)*eta  
          stdfsc(i)=dsqrt(2*Rb*t_bath/d_time)
        enddo 
        write (iout,'(/2a/)') 
     &   "Radii of site types and friction coefficients and std's of",
     &   " stochastic forces of fully exposed sites"
        write (iout,'(a5,f5.2,2f10.5)') 'p',pstok,gamp,stdfp*dsqrt(gamp)
        do i=1,ntyp
          write (iout,'(a5,f5.2,2f10.5)') restyp(i),restok(i),
     &     gamsc(i),stdfsc(i)*dsqrt(gamsc(i))
        enddo
      else if (tbf) then
        write (iout,'(a)') "Berendsen bath calculation"
        write (iout,'(a60,f10.5)') "Temperature:",t_bath
        write (iout,'(a60,f10.5)') "Coupling constant (tau):",tau_bath
        if (reset_moment) 
     &  write (iout,'(a,i10,a)') "Momenta will be reset at zero every",
     &   count_reset_moment," steps"
        if (reset_vel) 
     &    write (iout,'(a,i10,a)') 
     &    "Velocities will be reset at random every",count_reset_vel,
     &   " steps"
      else
        write (iout,'(a)') "Microcanonical mode calculation"
      endif
      if (rest) write (iout,'(/a/)') "===== Calculation restarted ===="
      write (iout,'(/30(1h=),a,29(1h=)/)') " End of MD run setup "
      return
      end
c------------------------------------------------------------------------------
      subroutine molread
C
C Read molecular data.
C
      include 'DIMENSIONS'
#ifdef MP
      include 'mpif.h'
#endif
      include 'COMMON.IOUNITS'
      include 'COMMON.GEO'
      include 'COMMON.VAR'
      include 'COMMON.INTERACT'
      include 'COMMON.LOCAL'
      include 'COMMON.NAMES'
      include 'COMMON.CHAIN'
      include 'COMMON.FFIELD'
      include 'COMMON.SBRIDGE'
      include 'COMMON.HEADER'
      include 'COMMON.CONTROL'
      include 'COMMON.DBASE'
      include 'COMMON.THREAD'
      include 'COMMON.CONTACTS'
      include 'COMMON.TORCNSTR'
      include 'COMMON.TIME1'
      include 'COMMON.BOUNDS'
      include 'COMMON.ZSCORE'
#ifdef MP
      include 'COMMON.INFO'
      integer error_msg
#endif
      character*4 sequence(maxres)
      integer rescode
      double precision x(maxvar)
      character*64 pdbfile
      character*320 weightcard
      character*80 weightcard_t,ucase
      dimension itype_pdb(maxres)
      common /pizda/ itype_pdb
      logical seq_comp,fail
      double precision energia(0:n_ene)
C
C Body
C
C Read weights of the subsequent energy terms.
      call card_concat(weightcard)
      call reada(weightcard,'WLONG',wlong,1.0D0)
      call reada(weightcard,'WSCC',wsc,wlong)
      call reada(weightcard,'WSCP',wscp,wlong)
      call reada(weightcard,'WELEC',welec,1.0D0)
      call reada(weightcard,'WVDWPP',wvdwpp,welec)
      call reada(weightcard,'WEL_LOC',wel_loc,1.0D0)
      call reada(weightcard,'WCORR4',wcorr4,0.0D0)
      call reada(weightcard,'WCORR5',wcorr5,0.0D0)
      call reada(weightcard,'WCORR6',wcorr6,0.0D0)
      call reada(weightcard,'WTURN3',wturn3,1.0D0)
      call reada(weightcard,'WTURN4',wturn4,1.0D0)
      call reada(weightcard,'WTURN6',wturn6,1.0D0)
      call reada(weightcard,'WSTRAIN',wstrain,1.0D0)
      call reada(weightcard,'WBOND',wbond,1.0D0)
      call reada(weightcard,'WTOR',wtor,1.0D0)
      call reada(weightcard,'WTORD',wtor_d,1.0D0)
      call reada(weightcard,'WANG',wang,1.0D0)
      call reada(weightcard,'WSCLOC',wscloc,1.0D0)
      call reada(weightcard,'SCAL14',scal14,0.4D0)
      call reada(weightcard,'SCALSCP',scalscp,1.0d0)
      call reada(weightcard,'CUTOFF',cutoff_corr,7.0d0)
      call reada(weightcard,'DELT_CORR',delt_corr,0.5d0)
      if (index(weightcard,'SOFT').gt.0) ipot=6
C 12/1/95 Added weight for the multi-body term WCORR
      call reada(weightcard,'WCORRH',wcorr,1.0D0)
      if (wcorr4.gt.0.0d0) wcorr=wcorr4
      weights(1)=wlong
      weights(2)=wscp
      weights(3)=welec
      weights(4)=wcorr
      weights(5)=wcorr5
      weights(6)=wcorr6
      weights(7)=wel_loc
      weights(8)=wturn3
      weights(9)=wturn4
      weights(10)=wturn6
      weights(11)=wang
      weights(12)=wscloc
      weights(13)=wtor
      weights(14)=wtor_d
      weights(15)=wstrain
      weights(16)=scal14
      write (iout,10) wsc,wscp,welec,wvdwpp,wbond,wang,wscloc,wtor,
     &  wtor_d,wstrain,wel_loc,wcorr,wcorr5,wcorr6,wturn3,
     &  wturn4,wturn6
   10 format (/'Energy-term weights:'//
     & 'WSCC=   ',f10.6,' (SC-SC)'/
     & 'WSCP=   ',f10.6,' (SC-p)'/
     & 'WELEC=  ',f10.6,' (p-p electr)'/
     & 'WVDWPP= ',f10.6,' (p-p VDW)'/
     & 'WBOND=  ',f10.6,' (stretching)'/
     & 'WANG=   ',f10.6,' (bending)'/
     & 'WSCLOC= ',f10.6,' (SC local)'/
     & 'WTOR=   ',f10.6,' (torsional)'/
     & 'WTORD=  ',f10.6,' (double torsional)'/
     & 'WSTRAIN=',f10.6,' (SS bridges & dist. cnstr.)'/
     & 'WEL_LOC=',f10.6,' (multi-body 3-rd order)'/
     & 'WCORR4= ',f10.6,' (multi-body 4th order)'/
     & 'WCORR5= ',f10.6,' (multi-body 5th order)'/
     & 'WCORR6= ',f10.6,' (multi-body 6th order)'/
     & 'WTURN3= ',f10.6,' (turns, 3rd order)'/
     & 'WTURN4= ',f10.6,' (turns, 4th order)'/
     & 'WTURN6= ',f10.6,' (turns, 6th order)')
      if (wcorr4.gt.0.0d0) then
        write (iout,'(/2a/)') 'Local-electrostatic type correlation ',
     &   'between contact pairs of peptide groups'
        write (iout,'(2(a,f5.3/))') 
     &  'Cutoff on 4-6th order correlation terms: ',cutoff_corr,
     &  'Range of quenching the correlation terms:',2*delt_corr 
      else if (wcorr.gt.0.0d0) then
        write (iout,'(/2a/)') 'Hydrogen-bonding correlation ',
     &   'between contact pairs of peptide groups'
      endif
      write (iout,'(a,f8.3)') 
     &  'Scaling factor of 1,4 SC-p interactions:',scal14
      write (iout,'(a,f8.3)') 
     &  'General scaling factor of SC-p interactions:',scalscp
      r0_corr=cutoff_corr-delt_corr
      do i=1,20
        aad(i,1)=scalscp*aad(i,1)
        aad(i,2)=scalscp*aad(i,2)
        bad(i,1)=scalscp*bad(i,1)
        bad(i,2)=scalscp*bad(i,2)
      enddo
c      print *,'indpdb=',indpdb,' pdbref=',pdbref
      if (indpdb.gt.0 .or. pdbref) then
        read(inp,'(a)') pdbfile
        write (iout,'(2a)') 'PDB data will be read from file ',pdbfile
        open(ipdbin,file=pdbfile,status='old',err=33)
        goto 34 
  33    write (iout,'(a)') 'Error opening PDB file.'
        stop
  34    continue
c        print *,'Begin reading pdb data'
        call readpdb
c        print *,'Finished reading pdb data'
        write (iout,'(a,i3,a,i3)')'nsup=',nsup,' nstart_sup=',nstart_sup
        do i=1,nres
          itype_pdb(i)=itype(i)
        enddo
        close (ipdbin)
        nnt=nstart_sup
        nct=nstart_sup+nres-1
        call contact(.true.,ncont_ref,icont_ref,co)

        if (sideadd) then
         write(iout,*)'Adding sidechains'
         maxsi=1000
         do i=2,nres-1
          iti=itype(i)
          if (iti.ne.10) then
            nsi=0
            fail=.true.
            do while (fail.and.nsi.le.maxsi)
              call gen_side(iti,theta(i+1),alph(i),omeg(i),fail)
              nsi=nsi+1
            enddo
            if(fail) write(iout,*)'Adding sidechain failed for res ',
     &              i,' after ',nsi,' trials'
          endif
         enddo
        endif  


      endif
      if (indpdb.eq.0) then
C Read sequence if not taken from the pdb file.
        read (inp,*) nres
c        print *,'nres=',nres
        if (iscode.gt.0) then
          read (inp,'(80a1)') (sequence(i)(1:1),i=1,nres)
        else
          read (inp,'(20(1x,a3))') (sequence(i),i=1,nres)
        endif
c ZSCORE
        if(modecalc.eq.9) then
         do i=1,nres
          seq_zs(i,1)=sequence(i)
         enddo
         nres_zs(1)=nres
        endif
C Convert sequence to numeric code
        do i=1,nres
          itype(i)=rescode(i,sequence(i),iscode)
        enddo
C Assign initial virtual bond lengths
        do i=2,nres
          vbld(i)=vbl
          vbld_inv(i)=vblinv
        enddo
        do i=2,nres-1
          vbld(i+nres)=dsc(itype(i))
          vbld_inv(i+nres)=dsc_inv(itype(i))
c          write (iout,*) "i",i," itype",itype(i),
c     &      " dsc",dsc(itype(i))," vbld",vbld(i),vbld(i+nres)
        enddo
      endif
c      print *,nres
c      print '(20i4)',(itype(i),i=1,nres)
      do i=1,nres
#ifdef PROCOR
        if (itype(i).eq.21 .or. itype(i+1).eq.21) then
#else
        if (itype(i).eq.21) then
#endif
          itel(i)=0
#ifdef PROCOR
        else if (itype(i+1).ne.20) then
#else
        else if (itype(i).ne.20) then
#endif
	  itel(i)=1
        else
	  itel(i)=2
        endif  
      enddo
c      print *,'Call Read_Bridge.'
      call read_bridge
C 8/13/98 Set limits to generating the dihedral angles
      do i=1,nres
        phibound(1,i)=-pi
        phibound(2,i)=pi
      enddo
      read (inp,*) ndih_constr
      if (ndih_constr.gt.0) then
        read (inp,*) ftors
        read (inp,*) (idih_constr(i),phi0(i),drange(i),i=1,ndih_constr)
        write (iout,*) 
     & 'There are',ndih_constr,' constraints on phi angles.'
        do i=1,ndih_constr
          write (iout,'(i5,2f8.3)') idih_constr(i),phi0(i),drange(i)
        enddo
        do i=1,ndih_constr
          phi0(i)=deg2rad*phi0(i)
          drange(i)=deg2rad*drange(i)
        enddo
        write (iout,*) 'FTORS',ftors
        do i=1,ndih_constr
          ii = idih_constr(i)
          phibound(1,ii) = phi0(i)-drange(i)
          phibound(2,ii) = phi0(i)+drange(i)
        enddo 
      endif
      nnt=1
      write (iout,'(a)') 'Boundaries in phi angle sampling:'
      do i=1,nres
        write (iout,'(a3,i5,2f10.1)') 
     &   restyp(itype(i)),i,phibound(1,i)*rad2deg,phibound(2,i)*rad2deg
      enddo
      nct=nres
c      print *,'NNT=',NNT,' NCT=',NCT
      if (itype(1).eq.21) nnt=2
      if (itype(nres).eq.21) nct=nct-1
      if (pdbref) then
        write (iout,'(a,i3)') 'nsup=',nsup
        nstart_seq=nnt
        if (nsup.le.(nct-nnt+1)) then
          do i=0,nct-nnt+1-nsup
            if (seq_comp(itype(nnt+i),itype_pdb(nstart_sup),nsup)) then
              nstart_seq=nnt+i
              goto 111
            endif
          enddo
          write (iout,'(a)') 
     &            'Error - sequences to be superposed do not match.'
          stop
        else
          do i=0,nsup-(nct-nnt+1)
            if (seq_comp(itype(nnt),itype_pdb(nstart_sup+i),nct-nnt+1)) 
     &      then
              nstart_sup=nstart_sup+i
              nsup=nct-nnt+1
              goto 111
            endif
          enddo 
          write (iout,'(a)') 
     &            'Error - sequences to be superposed do not match.'
        endif
  111   continue
        if (nsup.eq.0) nsup=nct-nnt
        if (nstart_sup.eq.0) nstart_sup=nnt
        if (nstart_seq.eq.0) nstart_seq=nnt
        write (iout,*) 'nsup=',nsup,' nstart_sup=',nstart_sup,
     &                 ' nstart_seq=',nstart_seq
      endif
c--- Zscore rms -------
      if (nz_start.eq.0) nz_start=nnt
      if (nz_end.eq.0 .and. nsup.gt.0) then
        nz_end=nnt+nsup-1
      else 
        nz_end=nct
      endif
      write (iout,*) 'NZ_START=',nz_start,' NZ_END=',nz_end
      write (iout,*) 'IZ_SC=',iz_sc
c----------------------
      call init_int_table
      if (refstr) then
        if (.not.pdbref) then
          call read_angles(inp,*38)
          goto 39
   38     write (iout,'(a)') 'Error reading reference structure.'
#ifdef MPL
          call mp_stopall(Error_Msg)
#else
          stop 'Error reading reference structure'
#endif
   39     call chainbuild
          call setup_var
          call geom_to_var(nvar,coord_exp_zs(1,1))
          nstart_sup=nnt
          nstart_seq=nnt
          nsup=nct-nnt+1
          do i=1,2*nres
            do j=1,3
              cref(j,i)=c(j,i)
            enddo
          enddo
          call contact(.true.,ncont_ref,icont_ref,co)
        endif
        write (iout,*) 'Contact order:',co
        if (pdbref) then
        write (2,*) 'Shifting contacts:',nstart_seq,nstart_sup
        do i=1,ncont_ref
          do j=1,2
            icont_ref(j,i)=icont_ref(j,i)+nstart_seq-nstart_sup
          enddo
          write (2,*) i,' ',restyp(itype(icont_ref(1,i))),' ',
     &     icont_ref(1,i),' ',
     &     restyp(itype(icont_ref(2,i))),' ',icont_ref(2,i)
        enddo
        endif
      endif
      if (indpdb.eq.0 .and. modecalc.ne.2 .and. modecalc.ne.4
     &    .and. modecalc.ne.8 .and. modecalc.ne.9 .and. 
     &    modecalc.ne.10) then
C If input structure hasn't been supplied from the PDB file read or generate
C initial geometry.
        if (iranconf.eq.0) then
          write (iout,'(a)') 'Initial geometry will be read in.'
          call read_angles(inp,*36)
          goto 37
   36     write (iout,'(a)') 'Error reading angle file.'
#ifdef MPL
	  call mp_stopall(error_msg)
#else
          stop 'Error reading angle file.'
#endif
   37     continue 
        else
          write (iout,'(a)') 'Random-generated initial geometry.'
#ifdef MP
c          print *,'MyID',MyID,' MasterID',MasterID
          if (MyID.eq.MasterID) then  
            do itrial=1,3
              itmp=1
              call gen_rand_conf(itmp,*30)
              goto 40
   30         write (iout,*) 'Failed to generate random conformation',
     &          ', itrial=',itrial
              write (*,*) 'Processor:',MyID,
     &          ' Failed to generate random conformation',
     &          ' itrial=',itrial
              call intout

#ifdef AIX
              call flush_(iout)
#else
              call flush(iout)
#endif
            enddo
            write (iout,'(a,i3,a)') 'Processor:',MyID,
     &        ' error in generating random conformation.'
            write (*,'(a,i3,a)') 'Processor:',MyID,
     &        ' error in generating random conformation.'
#ifdef MPI
            call mpi_abort(mpi_comm_world,error_msg,ierrcode)            
#else
	    call mp_stopall(error_msg)
#endif         
   40       continue
#ifdef MPL
            if (fgProcs.gt.1) then
              msglen=maxres*4*8
              call mp_bcast(theta(1),msglen,BossID,fgGroupID)
              if (MyID.ne.BossID) call chainbuild
            endif
#endif
          endif
#else
          do itrial=1,100
            itmp=1
            call gen_rand_conf(itmp,*30)
            goto 40
   30       write (iout,*) 'Failed to generate random conformation',
     &        ', itrial=',itrial
            write (*,*) 'Failed to generate random conformation',
     &        ', itrial=',itrial
          enddo
          write (iout,'(a,i3,a)') 'Processor:',MyID,
     &      ' error in generating random conformation.'
          write (*,'(a,i3,a)') 'Processor:',MyID,
     &      ' error in generating random conformation.'
          stop
   40     continue
#endif
        endif
      elseif (modecalc.eq.4) then
        read (inp,'(a)') intinname
        open (intin,file=intinname,status='old',err=333)
        write (iout,'(a)') 'intinname',intinname
        write (*,'(a)') 'intinname',intinname
        goto 334
  333   write (iout,'(2a)') 'Error opening angle file ',intinname
#ifdef MPL 
        call mp_stopall(Error_Msg)
#else
        stop 'Error opening angle file.' 
#endif   
  334   continue
      endif
C Generate distance constraints, if the PDB structure is to be regularized. 
      if (nthread.gt.0) then
        call read_threadbase
      endif
      call setup_var
      call intout
      if (ns.gt.0) then
        write (iout,'(/a,i3,a)') 
     &  'The chain contains',ns,' disulfide-bridging cysteines.'
        write (iout,'(20i4)') (iss(i),i=1,ns)
        write (iout,'(/a/)') 'Pre-formed links are:' 
	do i=1,nss
	  i1=ihpb(i)-nres
	  i2=jhpb(i)-nres
	  it1=itype(i1)
	  it2=itype(i2)
	  write (iout,'(2a,i3,3a,i3,a,3f10.3)')
     &    restyp(it1),'(',i1,') -- ',restyp(it2),'(',i2,')',dhpb(i),
     &    ebr,forcon(i)
	enddo
	write (iout,'(a)')
      endif
      if (i2ndstr.gt.0) call secstrp2dihc
c      call geom_to_var(nvar,x)
c      call etotal(energia(0))
c      call enerprint(energia(0))
c      call briefout(0,etot)
c      stop
cd    write (iout,'(2(a,i3))') 'NNT',NNT,' NCT',NCT
cd    write (iout,'(a)') 'Variable list:'
cd    write (iout,'(i4,f10.5)') (i,rad2deg*x(i),i=1,nvar)
#ifdef MPL
      write (iout,'(//80(1h*)/20x,a,i4,a/80(1h*)//)') 
     &  'Processor',MyId,': end reading molecular data.'
#endif
      return
      end
c--------------------------------------------------------------------------
      subroutine molread_zs(molnum)
C
C Read molecular data.
C
      include 'DIMENSIONS'
      include 'COMMON.IOUNITS'
      include 'COMMON.GEO'
      include 'COMMON.VAR'
      include 'COMMON.INTERACT'
      include 'COMMON.LOCAL'
      include 'COMMON.NAMES'
      include 'COMMON.CHAIN'
      include 'COMMON.FFIELD'
      include 'COMMON.SBRIDGE'
      include 'COMMON.HEADER'
      include 'COMMON.CONTROL'
      include 'COMMON.DBASE'
      include 'COMMON.THREAD'
      include 'COMMON.CONTACTS'
      include 'COMMON.TORCNSTR'
      include 'COMMON.TIME1'
      include 'COMMON.BOUNDS'
      include 'COMMON.ZSCORE'
#ifdef MPL
      include 'COMMON.INFO'
#endif
      character*4 sequence(maxres)
      integer rescode
      double precision x(maxvar)
      character*64 pdbfile
      character*320 weightcard
      character*80 weightcard_t,ucase
      dimension itype_pdb(maxres)
      logical seq_comp
C
C Body
C
C Read sequence if not taken from the pdb file.
        imol_curr=molnum
        nres=nres_zs(molnum)
c        print *,'nres=',nres
c        print *, 'worker ',me,' molread_zs called for ',molnum
        do i=1, nres
         sequence(i)=seq_zs(i,molnum)
        enddo
C Convert sequence to numeric code
        do i=1,nres
          itype(i)=rescode(i,sequence(i),iscode)
        enddo
c      print *,nres
c      print '(20i4)',(itype(i),i=1,nres)
      do i=1,nres
	if (itype(i).ne.20) then
	  itel(i)=1
        else
	  itel(i)=2
        endif  
      enddo
C 8/13/98 Set limits to generating the dihedral angles
      do i=1,nres
        phibound(1,i)=-pi
        phibound(2,i)=pi
      enddo
      nnt=1
c      write (iout,'(a)') 'Boundaries in phi angle sampling:'
c      do i=1,nres
c        write (iout,'(a3,i5,2f10.1)') 
c     &   restyp(itype(i)),i,phibound(1,i)*rad2deg,phibound(2,i)*rad2deg
c      enddo
      nct=nres
c      print *,'NNT=',NNT,' NCT=',NCT
      if (itype(1).eq.21) nnt=2
      if (itype(nres).eq.21) nct=nct-1
      call init_int_table
      call setup_var
      if (ns.gt.0) then
c        write (iout,'(/a,i3,a)') 
c     &  'The chain contains',ns,' disulfide-bridging cysteines.'
c        write (iout,'(20i4)') (iss(i),i=1,ns)
c        write (iout,'(/a/)') 'Pre-formed links are:' 
	do i=1,nss
	  i1=ihpb(i)-nres
	  i2=jhpb(i)-nres
	  it1=itype(i1)
	  it2=itype(i2)
c         write (iout,'(2a,i3,3a,i3,a,3f10.3)')
c     &    restyp(it1),'(',i1,') -- ',restyp(it2),'(',i2,')',dhpb(i),
c     &    ebr,forcon(i)
	enddo
c       write (iout,'(a)')
      endif
      return
      end
c-----------------------------------------------------------------------------
      logical function seq_comp(itypea,itypeb,length)
      implicit none
      integer length,itypea(length),itypeb(length)
      integer i
      do i=1,length
        if (itypea(i).ne.itypeb(i)) then
          seq_comp=.false.
          return
        endif
      enddo
      seq_comp=.true.
      return
      end
c-----------------------------------------------------------------------------
      subroutine read_bridge
C Read information about disulfide bridges.
      include 'DIMENSIONS'
#ifdef MP
      include 'mpif.h'
#endif
      include 'COMMON.IOUNITS'
      include 'COMMON.GEO'
      include 'COMMON.VAR'
      include 'COMMON.INTERACT'
      include 'COMMON.LOCAL'
      include 'COMMON.NAMES'
      include 'COMMON.CHAIN'
      include 'COMMON.FFIELD'
      include 'COMMON.SBRIDGE'
      include 'COMMON.HEADER'
      include 'COMMON.CONTROL'
      include 'COMMON.DBASE'
      include 'COMMON.THREAD'
      include 'COMMON.TIME1'
#ifdef MPL
      include 'COMMON.INFO'
#endif
C Read bridging residues.
      read (inp,*) ns,(iss(i),i=1,ns)
      print *,'ns=',ns
      write (iout,*) 'ns=',ns,' iss:',(iss(i),i=1,ns)
C Check whether the specified bridging residues are cystines.
      do i=1,ns
	if (itype(iss(i)).ne.1) then
	  write (iout,'(2a,i3,a)') 
     &   'Do you REALLY think that the residue ',restyp(iss(i)),i,
     &   ' can form a disulfide bridge?!!!'
	  write (*,'(2a,i3,a)') 
     &   'Do you REALLY think that the residue ',restyp(iss(i)),i,
     &   ' can form a disulfide bridge?!!!'
#ifdef MPL
	 call mp_stopall(error_msg)
#else
         stop
#endif
        endif
      enddo
C Read preformed bridges.
      if (ns.gt.0) then
      read (inp,*) nss,(ihpb(i),jhpb(i),i=1,nss)
      write (iout,*) 'nss=',nss,' ihpb,jhpb: ',(ihpb(i),jhpb(i),i=1,nss)
      if (nss.gt.0) then
        nhpb=nss
C Check if the residues involved in bridges are in the specified list of
C bridging residues.
        do i=1,nss
          do j=1,i-1
	    if (ihpb(i).eq.ihpb(j).or.ihpb(i).eq.jhpb(j)
     &      .or.jhpb(i).eq.ihpb(j).or.jhpb(i).eq.jhpb(j)) then
	      write (iout,'(a,i3,a)') 'Disulfide pair',i,
     &      ' contains residues present in other pairs.'
	      write (*,'(a,i3,a)') 'Disulfide pair',i,
     &      ' contains residues present in other pairs.'
#ifdef MPL
	      call mp_stopall(error_msg)
#else
              stop 
#endif
	    endif
          enddo
	  do j=1,ns
	    if (ihpb(i).eq.iss(j)) goto 10
          enddo
          write (iout,'(a,i3,a)') 'Pair',i,' contains unknown cystine.'
   10     continue
	  do j=1,ns
	    if (jhpb(i).eq.iss(j)) goto 20
          enddo
          write (iout,'(a,i3,a)') 'Pair',i,' contains unknown cystine.'
   20     continue
          dhpb(i)=dbr
          forcon(i)=fbr
        enddo
        do i=1,nss
          ihpb(i)=ihpb(i)+nres
          jhpb(i)=jhpb(i)+nres
        enddo
      endif
      endif
      return
      end
c----------------------------------------------------------------------------
      subroutine read_threadbase
      include 'DIMENSIONS'
      include 'COMMON.IOUNITS'
      include 'COMMON.GEO'
      include 'COMMON.VAR'
      include 'COMMON.INTERACT'
      include 'COMMON.LOCAL'
      include 'COMMON.NAMES'
      include 'COMMON.CHAIN'
      include 'COMMON.FFIELD'
      include 'COMMON.SBRIDGE'
      include 'COMMON.HEADER'
      include 'COMMON.CONTROL'
      include 'COMMON.DBASE'
      include 'COMMON.THREAD'
      include 'COMMON.TIME1'
C Read pattern database for threading.
      read (icbase,*) nseq
      do i=1,nseq
        read (icbase,'(i5,2x,a8,2i4)') nres_base(1,i),str_nam(i),
     &   nres_base(2,i),nres_base(3,i)
        read (icbase,'(9f8.3)') ((cart_base(k,j,i),k=1,3),j=1,
     &   nres_base(1,i))
c       write (iout,'(i5,2x,a8,2i4)') nres_base(1,i),str_nam(i),
c    &   nres_base(2,i),nres_base(3,i)
c       write (iout,'(9f8.3)') ((cart_base(k,j,i),k=1,3),j=1,
c    &   nres_base(1,i))
      enddo
      close (icbase)
      if (weidis.eq.0.0D0) weidis=0.1D0
      do i=nnt,nct
        do j=i+2,nct
          nhpb=nhpb+1
          ihpb(nhpb)=i
          jhpb(nhpb)=j
          forcon(nhpb)=weidis
        enddo
      enddo 
      read (inp,*) nexcl,(iexam(1,i),iexam(2,i),i=1,nexcl)
      write (iout,'(a,i5)') 'nexcl: ',nexcl
      write (iout,'(2i5)') (iexam(1,i),iexam(2,i),i=1,nexcl)
      return
      end
c------------------------------------------------------------------------------
      subroutine setup_var
      include 'DIMENSIONS'
      include 'COMMON.IOUNITS'
      include 'COMMON.GEO'
      include 'COMMON.VAR'
      include 'COMMON.INTERACT'
      include 'COMMON.LOCAL'
      include 'COMMON.NAMES'
      include 'COMMON.CHAIN'
      include 'COMMON.FFIELD'
      include 'COMMON.SBRIDGE'
      include 'COMMON.HEADER'
      include 'COMMON.CONTROL'
      include 'COMMON.DBASE'
      include 'COMMON.THREAD'
      include 'COMMON.TIME1'
C Set up variable list.
      ntheta=nres-2
      nphi=nres-3
      nvar=ntheta+nphi
      nside=0
      do i=2,nres-1
        if (itype(i).ne.10) then
	  nside=nside+1
          ialph(i,1)=nvar+nside
	  ialph(nside,2)=i
        endif
      enddo
      if (indphi.gt.0) then
        nvar=nphi
      else if (indback.gt.0) then
        nvar=nphi+ntheta
      else
        nvar=nvar+2*nside
      endif
cd    write (iout,'(3i4)') (i,ialph(i,1),ialph(i,2),i=2,nres-1)
      return
      end
c----------------------------------------------------------------------------
      subroutine gen_dist_constr
C Generate CA distance constraints.
      include 'DIMENSIONS'
      include 'COMMON.IOUNITS'
      include 'COMMON.GEO'
      include 'COMMON.VAR'
      include 'COMMON.INTERACT'
      include 'COMMON.LOCAL'
      include 'COMMON.NAMES'
      include 'COMMON.CHAIN'
      include 'COMMON.FFIELD'
      include 'COMMON.SBRIDGE'
      include 'COMMON.HEADER'
      include 'COMMON.CONTROL'
      include 'COMMON.DBASE'
      include 'COMMON.THREAD'
      include 'COMMON.TIME1'
      dimension itype_pdb(maxres)
      common /pizda/ itype_pdb
      character*2 iden
cd      print *,'gen_dist_constr: nnt=',nnt,' nct=',nct
cd      write (2,*) 'gen_dist_constr: nnt=',nnt,' nct=',nct,
cd     & ' nstart_sup',nstart_sup,' nstart_seq',nstart_seq,
cd     & ' nsup',nsup
      do i=nstart_sup,nstart_sup+nsup-1
cd      write (2,*) 'i',i,' seq ',restyp(itype(i+nstart_seq-nstart_sup)),
cd     &    ' seq_pdb', restyp(itype_pdb(i))
        do j=i+2,nstart_sup+nsup-1
          nhpb=nhpb+1
          ihpb(nhpb)=i+nstart_seq-nstart_sup
          jhpb(nhpb)=j+nstart_seq-nstart_sup
          forcon(nhpb)=weidis
          dhpb(nhpb)=dist(i,j)
        enddo
      enddo 
cd      write (iout,'(a)') 'Distance constraints:' 
cd      do i=nss+1,nhpb
cd        ii=ihpb(i)
cd        jj=jhpb(i)
cd        iden='CA'
cd        if (ii.gt.nres) then
cd          iden='SC'
cd          ii=ii-nres
cd          jj=jj-nres
cd        endif
cd        write (iout,'(a,1x,a,i4,3x,a,1x,a,i4,2f10.3)') 
cd     &  restyp(itype(ii)),iden,ii,restyp(itype(jj)),iden,jj,
cd     &  dhpb(i),forcon(i)
cd      enddo
      return
      end
c----------------------------------------------------------------------------
      subroutine map_read
      include 'DIMENSIONS'
      include 'COMMON.MAP'
      include 'COMMON.IOUNITS'
      character*3 angid(4) /'THE','PHI','ALP','OME'/
      character*80 mapcard,ucase
      do imap=1,nmap
        read (inp,'(a)') mapcard
        mapcard=ucase(mapcard)
        if (index(mapcard,'PHI').gt.0) then
          kang(imap)=1
        else if (index(mapcard,'THE').gt.0) then
          kang(imap)=2
        else if (index(mapcard,'ALP').gt.0) then
          kang(imap)=3
        else if (index(mapcard,'OME').gt.0) then
          kang(imap)=4
        else
          write(iout,'(a)')'Error - illegal variable spec in MAP card.'
          stop 'Error - illegal variable spec in MAP card.'
        endif
        call readi (mapcard,'RES1',res1(imap),0)
        call readi (mapcard,'RES2',res2(imap),0)
        if (res1(imap).eq.0) then
          res1(imap)=res2(imap)
        else if (res2(imap).eq.0) then
          res2(imap)=res1(imap)
        endif
        if(res1(imap)*res2(imap).eq.0 .or. res1(imap).gt.res2(imap))then
          write (iout,'(a)') 
     &    'Error - illegal definition of variable group in MAP.'
          stop 'Error - illegal definition of variable group in MAP.'
        endif
        call reada(mapcard,'FROM',ang_from(imap),0.0D0)
        call reada(mapcard,'TO',ang_to(imap),0.0D0)
        call readi(mapcard,'NSTEP',nstep(imap),0)
        if (ang_from(imap).eq.ang_to(imap) .or. nstep(imap).eq.0) then
          write (iout,'(a)') 
     &     'Illegal boundary and/or step size specification in MAP.'
          stop 'Illegal boundary and/or step size specification in MAP.'
        endif
      enddo ! imap
      return
      end 
c----------------------------------------------------------------------------
      subroutine csaread
      include 'DIMENSIONS'
      include 'COMMON.ZSCORE'
      include 'COMMON.IOUNITS'
      include 'COMMON.GEO'
      include 'COMMON.CSA'
      include 'COMMON.BANK'
      character*80 ucase
      character*620 mcmcard
      call card_concat(mcmcard)

      call readi(mcmcard,'NCONF',nconf,50)
      call readi(mcmcard,'JSTART',jstart,1)
      call readi(mcmcard,'JEND',jend,1)
      call readi(mcmcard,'NSTMAX',nstmax,500000)
      call readi(mcmcard,'N1',n1,6)
      call readi(mcmcard,'N2',n2,4)
      call readi(mcmcard,'N3',n3,0)
      call readi(mcmcard,'N4',n4,0)
      call readi(mcmcard,'N5',n5,0)
      call readi(mcmcard,'N6',n6,10)
      call readi(mcmcard,'N7',n7,0)
      call readi(mcmcard,'N8',n8,0)
      call readi(mcmcard,'NDIFF',ndiff,4)
      call readi(mcmcard,'IS1',is1,1)
      call readi(mcmcard,'IS2',is2,8)
      call readi(mcmcard,'NRAN0',nran0,4)
      call readi(mcmcard,'NRAN1',nran1,2)
      call readi(mcmcard,'IRR',irr,1)
      call readi(mcmcard,'NSEED',nseed,20)
      call readi(mcmcard,'NTOTAL',ntotal,10000)
      call reada(mcmcard,'CUT1',cut1,2.0d0)
      call reada(mcmcard,'CUT2',cut2,5.0d0)
      call reada(mcmcard,'ESTOP',estop,-3000.0d0)
      call readi(mcmcard,'ICMAX',icmax,3)
      call readi(mcmcard,'IRESTART',irestart,0)
      call readi(mcmcard,'NBANKTM',ntbankm,100)
      call reada(mcmcard,'DELE',dele,20.0d0)
      call reada(mcmcard,'DIFCUT',difcut,720.0d0)
      call readi(mcmcard,'IREF',iref,0)
      call reada(mcmcard,'RMSCUT',rmscut,4.0d0)
      call reada(mcmcard,'PNCCUT',pnccut,0.5d0)
      call readi(mcmcard,'NCONF_IN',nconf_in,0)
      call reada(mcmcard,'RDIH_BIAS',rdih_bias,0.5d0)
      write (iout,*) "NCONF_IN",nconf_in
      return
      end
c----------------------------------------------------------------------------
      subroutine mcmfread
      include 'DIMENSIONS'
      include 'COMMON.MCMF'
      include 'COMMON.IOUNITS'
      include 'COMMON.GEO'
      character*80 ucase
      character*620 mcmcard
      call card_concat(mcmcard)

      call readi(mcmcard,'MAXRANT',maxrant,1000)
      write(iout,*)'MAXRANT=',maxrant
      call readi(mcmcard,'MAXFAM',maxfam,maxfam_p)
      write(iout,*)'MAXFAM=',maxfam
      call readi(mcmcard,'NNET1',nnet1,5)
      write(iout,*)'NNET1=',nnet1
      call readi(mcmcard,'NNET2',nnet2,4)
      write(iout,*)'NNET2=',nnet2
      call readi(mcmcard,'NNET3',nnet3,4)
      write(iout,*)'NNET3=',nnet3
      call readi(mcmcard,'ILASTT',ilastt,0)
      write(iout,*)'ILASTT=',ilastt
      call readi(mcmcard,'MAXSTR',maxstr,maxstr_mcmf)
      write(iout,*)'MAXSTR=',maxstr
      maxstr_f=maxstr/maxfam
      write(iout,*)'MAXSTR_F=',maxstr_f
      call readi(mcmcard,'NMCMF',nmcmf,10)
      write(iout,*)'NMCMF=',nmcmf
      call readi(mcmcard,'IFOCUS',ifocus,nmcmf)
      write(iout,*)'IFOCUS=',ifocus
      call readi(mcmcard,'NLOCMCMF',nlocmcmf,1000)
      write(iout,*)'NLOCMCMF=',nlocmcmf
      call readi(mcmcard,'INTPRT',intprt,1000)
      write(iout,*)'INTPRT=',intprt
      call readi(mcmcard,'IPRT',iprt,100)
      write(iout,*)'IPRT=',iprt
      call readi(mcmcard,'IMAXTR',imaxtr,100)
      write(iout,*)'IMAXTR=',imaxtr
      call readi(mcmcard,'MAXEVEN',maxeven,1000)
      write(iout,*)'MAXEVEN=',maxeven
      call readi(mcmcard,'MAXEVEN1',maxeven1,3)
      write(iout,*)'MAXEVEN1=',maxeven1
      call readi(mcmcard,'INIMIN',inimin,200)
      write(iout,*)'INIMIN=',inimin
      call readi(mcmcard,'NSTEPMCMF',nstepmcmf,10)
      write(iout,*)'NSTEPMCMF=',nstepmcmf
      call readi(mcmcard,'NTHREAD',nthread,5)
      write(iout,*)'NTHREAD=',nthread
      call readi(mcmcard,'MAXSTEPMCMF',maxstepmcmf,2500)
      write(iout,*)'MAXSTEPMCMF=',maxstepmcmf
      call readi(mcmcard,'MAXPERT',maxpert,9)
      write(iout,*)'MAXPERT=',maxpert
      call readi(mcmcard,'IRMSD',irmsd,1)
      write(iout,*)'IRMSD=',irmsd
      call reada(mcmcard,'DENEMIN',denemin,0.01D0)
      write(iout,*)'DENEMIN=',denemin
      call reada(mcmcard,'RCUT1S',rcut1s,3.5D0)
      write(iout,*)'RCUT1S=',rcut1s
      call reada(mcmcard,'RCUT1E',rcut1e,2.0D0)
      write(iout,*)'RCUT1E=',rcut1e
      call reada(mcmcard,'RCUT2S',rcut2s,0.5D0)
      write(iout,*)'RCUT2S=',rcut2s
      call reada(mcmcard,'RCUT2E',rcut2e,0.1D0)
      write(iout,*)'RCUT2E=',rcut2e
      call reada(mcmcard,'DPERT1',d_pert1,180.0D0)
      write(iout,*)'DPERT1=',d_pert1
      call reada(mcmcard,'DPERT1A',d_pert1a,180.0D0)
      write(iout,*)'DPERT1A=',d_pert1a
      call reada(mcmcard,'DPERT2',d_pert2,90.0D0)
      write(iout,*)'DPERT2=',d_pert2
      call reada(mcmcard,'DPERT2A',d_pert2a,45.0D0)
      write(iout,*)'DPERT2A=',d_pert2a
      call reada(mcmcard,'DPERT2B',d_pert2b,90.0D0)
      write(iout,*)'DPERT2B=',d_pert2b
      call reada(mcmcard,'DPERT2C',d_pert2c,60.0D0)
      write(iout,*)'DPERT2C=',d_pert2c
      d_pert1=deg2rad*d_pert1
      d_pert1a=deg2rad*d_pert1a
      d_pert2=deg2rad*d_pert2
      d_pert2a=deg2rad*d_pert2a
      d_pert2b=deg2rad*d_pert2b
      d_pert2c=deg2rad*d_pert2c
      call reada(mcmcard,'KT_MCMF1',kt_mcmf1,1.0D0)
      write(iout,*)'KT_MCMF1=',kt_mcmf1
      call reada(mcmcard,'KT_MCMF2',kt_mcmf2,1.0D0)
      write(iout,*)'KT_MCMF2=',kt_mcmf2
      call reada(mcmcard,'DKT_MCMF1',dkt_mcmf1,10.0D0)
      write(iout,*)'DKT_MCMF1=',dkt_mcmf1
      call reada(mcmcard,'DKT_MCMF2',dkt_mcmf2,1.0D0)
      write(iout,*)'DKT_MCMF2=',dkt_mcmf2
      call reada(mcmcard,'RCUTINI',rcutini,3.5D0)
      write(iout,*)'RCUTINI=',rcutini
      call reada(mcmcard,'GRAT',grat,0.5D0)
      write(iout,*)'GRAT=',grat
      call reada(mcmcard,'BIAS_MCMF',bias_mcmf,0.0D0)
      write(iout,*)'BIAS_MCMF=',bias_mcmf

      return
      end 
c----------------------------------------------------------------------------
      subroutine zsread
      include 'DIMENSIONS'
      include 'COMMON.ZSCORE'
      include 'COMMON.IOUNITS'
      include 'COMMON.GEO'
      character*80 ucase
      character*620 mcmcard
      call card_concat(mcmcard)
      call readi(mcmcard,'IPRT',icntrl(4),1)
      call readi(mcmcard,'IMN',icntrl(1),1)
      call readi(mcmcard,'NDD',icntrl(3),1)
      call readi(mcmcard,'ISPECMAX',icntrl(6),2)
      icntrl(5)=iout
      call reada(mcmcard,'DELTA',rcntrl(1),0.02D0)
      call reada(mcmcard,'DDELTA',rcntrl(2),0.10D0)
      call reada(mcmcard,'DOK',rcntrl(3),1.0D-3)
      call reada(mcmcard,'DOK2',rcntrl(4),0.01D-5)
      call reada(mcmcard,'DOK3',rcntrl(5),1.0D0)
      strict_zs=(index(mcmcard,'STRICT').gt.0)
      t_strict_zs=(index(mcmcard,'T_STRICT').gt.0)
      pert_search=(index(mcmcard,'PERT_SEARCH').gt.0)
      mc_all=(index(mcmcard,'MC_ALL').gt.0)
      inicsa=(index(mcmcard,'INICSA').gt.0)
      ini_clu=(index(mcmcard,'INI_CLU').gt.0)
      wcorr6_zs=(index(mcmcard,'WCORR6').gt.0)
      call reada(mcmcard,'RMS_CUT',rms_cut,2.0D0)
      call reada(mcmcard,'ZS_ZTOL',zs_ztol,0.25d0)
      call reada(mcmcard,'DBACK',dback,0.1d0)
      call reada(mcmcard,'RMS_CUT1',rms_cut1,rms_cut)
      call reada(mcmcard,'RMS_CUT1',rms_cut2,rms_cut)
      call reada(mcmcard,'WE_MIN',we_min,1.0D-4)
      call reada(mcmcard,'DANN',dann,0.1D0)
      call reada(mcmcard,'W_Z',w_zl,1.0D0)
      call reada(mcmcard,'RKTZ_Z',rktz_z,1.0D0)
      call reada(mcmcard,'RKTZ_R',rktz_r,1.0D0)
      call reada(mcmcard,'RKTZ_RA',rktz_ra,1.0D0)
      call reada(mcmcard,'PERT_Z',pert_z,0.1D0)
      call reada(mcmcard,'W_R',w_rrl,1.0D0)
      call reada(mcmcard,'DELTA_ZS',delta_zs,2.0D-3)
      call reada(mcmcard,'D_PERT',d_pert,20.0d0)
      call readi(mcmcard,'ITERMAXZS',itermaxzs,100)
      call readi(mcmcard,'IANNEAL',ianneal,0)
      call readi(mcmcard,'ITERL',icntrl(2),100)
      call readi(mcmcard,'IZTYPE',iztype,0)
      call readi(mcmcard,'LEVELZS',levelzs,3)
      call readi(mcmcard,'NPROT',nprot_zs,1)
      call readi(mcmcard,'IPERT_P',ipert_p,0)
      call readi(mcmcard,'NNNT_REP',nnnt_rep,1)
      call readi(mcmcard,'MCMAX_ZS',mcmax_zs,100)
      call readi(mcmcard,'MCMAXT_ZS',mcmaxt_zs,300)
      call readi(mcmcard,'ITERR_ZS',iterr_zs,10)
      call readi(mcmcard,'IPERT',ipert,1)
      call readi(mcmcard,'ICONT_ZS',icont_zs,0)
      call readi(mcmcard,'MIN_ZS',min_zs,1)
      call readi(mcmcard,'IDIST_ZS',idist_zs,1)
      call reada(mcmcard,'TOLF',tolf_zs,1.0D-4)
      call reada(mcmcard,'RTOLF',rtolf_zs,1.0D-4)
      call reada(mcmcard,'ACCR_MIN',accr_min,0.35D0)
      call reada(mcmcard,'ACCR_MAX',accr_max,0.65D0)
      call reada(mcmcard,'DETT',dett,1.5D0)
      call readi(mcmcard,'IACCR',iaccr,20)
      call readi(mcmcard,'IZSMIN',izsmin,0)
      call readi(mcmcard,'NZSCMAX',nzscmax,maxzs1)
      call readi(mcmcard,'NZSMAX',nzsmax,maxzs)
      call readi(mcmcard,'NNAT_ZS',nnat_zs,nzsmax/2)
      call readi(mcmcard,'NNATC_ZS',nnatc_zs,nzscmax/2)
      call readi(mcmcard,'NZS_CSA',nzs_csa,3)
      call readi(mcmcard,'IBACKTRK',ibacktrk,0)
c      call readi(mcmcard,'NZ_START',nz_start,0)
c      call readi(mcmcard,'NZ_END',nz_end,0)
c      call readi(mcmcard,'IZ_SC',iz_sc,0)


      write(iout,*)'ZSCORE input parameters'
      write(iout,*)'IPRT=',icntrl(4)
      write(iout,*)'IMN=',icntrl(1)
      write(iout,*)'ITER=',icntrl(2)
      write(iout,*)'NDD=',icntrl(3)
      write(iout,*)'ISPECMAX=',icntrl(6)
      write(iout,*)'DELTA=',rcntrl(1)
      write(iout,*)'DDELTA=',rcntrl(2)
      write(iout,*)'DOK=',rcntrl(3)
      write(iout,*)'DOK2=',rcntrl(4)
      write(iout,*)'DOK3=',rcntrl(5)
      write(iout,*)'STRICT=',strict_zs
      write(iout,*)'T_STRICT=',t_strict_zs
      write(iout,*)'WCORR6=',wcorr6_zs
      write(iout,*)'MC_ALL=',mc_all
      write(iout,*)'INICSA=',inicsa
      write(iout,*)'INI_CLU=',ini_clu
      write(iout,*)'LEVELZS=',levelzs
      write(iout,*)'NPROT=',nprot_zs
      write(iout,*)'IPERT_P=',ipert_p
      write(iout,*)'ipert=',ipert
      write(iout,*)'icont_zs=',icont_zs
      write(iout,*)'mcmax_zs=',mcmax_zs
      write(iout,*)'mcmaxt_zs=',mcmaxt_zs
      write(iout,*)'iterr_zs=',iterr_zs
      write(iout,*)'pert_search=',pert_search
      write(iout,*)'MIN_ZS=',min_zs
      write(iout,*)'IDIST_ZS=',idist_zs
      write(iout,*)'D_PERT=',d_pert
      d_pert=deg2rad*d_pert
      write(iout,*)'D_PERT(rad)=',d_pert
      write(iout,*)'ZS_ZTOL=',zs_ztol
      write(iout,*)'RMS_CUT=',rms_cut
      write(iout,*)'RMS_CUT1=',rms_cut1
      write(iout,*)'RMS_CUT2=',rms_cut2
      write(iout,*)'DANN=',dann
      write(iout,*)'IANNEAL=',ianneal
      write(iout,*)'RKTZ_Z=',rktz_z
      write(iout,*)'RKTZ_R=',rktz_r
      write(iout,*)'RKTZ_RA=',rktz_ra
      write(iout,*)'PERT_Z=',pert_z
      write(iout,*)'W_Z=',w_zl
      write(iout,*)'W_R=',w_rrl
      write(iout,*)'ITERMAXZS=',itermaxzs
      write(iout,*)'IZTYPE=',iztype
      write(iout,*)'DELTA_ZS=',DELTA_ZS
      write(iout,*)'TOLF=',tolf_zs
      write(iout,*)'RTOLF=',rtolf_zs
      write(iout,*)'ACCR_MIN=',accr_min
      write(iout,*)'ACCR_MAX=',accr_max
      write(iout,*)'IACCR=',iaccr
      write(iout,*)'DETT=',dett
      write(iout,*)'DBACK=',dback
      write(iout,*)'IZSMIN=',izsmin
      write(iout,*)'NZSCMAX=',nzscmax
      write(iout,*)'NZSMAX=',nzsmax
      write(iout,*)'NNNT_REP=',nnnt_rep
      write(iout,*)'NNAT_ZS=',nnat_zs
      write(iout,*)'NNATC_ZS=',nnatc_zs
      write(iout,*)'NZS_CSA=',nzs_csa
      write(iout,*)'IBACKTRK=',ibacktrk
      write(iout,*)'NZ_START=',nz_start,' NZ_END=',nz_end
      write(iout,*)'IZ_SC=',iz_sc

      do i=1, nprot_zs
       w_z(i)=w_zl
       w_rr(i)=w_rrl
       w_mol(i)=1.0d0
      enddo

      return
      end 
c----------------------------------------------------------------------------
      subroutine mcmread
      include 'DIMENSIONS'
      include 'COMMON.MCM'
      include 'COMMON.MCE'
      include 'COMMON.IOUNITS'
      character*80 ucase
      character*320 mcmcard
      call card_concat(mcmcard)
      call readi(mcmcard,'MAXACC',maxacc,100)
      call readi(mcmcard,'MAX_MCM_IT',max_mcm_it,10000)
      call readi(mcmcard,'MAXTRIAL',maxtrial,100)
      call readi(mcmcard,'MAXTRIAL_ITER',maxtrial_iter,1000)
      call readi(mcmcard,'MAXREPM',maxrepm,200)
      call reada(mcmcard,'RANFRACT',RanFract,0.5D0)
      call reada(mcmcard,'POOL_FRACT',pool_fraction,0.01D0)
      call reada(mcmcard,'OVERLAP',overlap_cut,1.0D3)
      call reada(mcmcard,'E_UP',e_up,5.0D0)
      call reada(mcmcard,'DELTE',delte,0.1D0)
      call readi(mcmcard,'NSWEEP',nsweep,5)
      call readi(mcmcard,'NSTEPH',nsteph,0)
      call readi(mcmcard,'NSTEPC',nstepc,0)
      call reada(mcmcard,'TMIN',tmin,298.0D0)
      call reada(mcmcard,'TMAX',tmax,298.0D0)
      call readi(mcmcard,'NWINDOW',nwindow,0)
      call readi(mcmcard,'PRINT_MC',print_mc,0)
      print_stat=(index(mcmcard,'NO_PRINT_STAT').le.0)
      print_int=(index(mcmcard,'NO_PRINT_INT').le.0)
      ent_read=(index(mcmcard,'ENT_READ').gt.0)
      call readi(mcmcard,'SAVE_FREQ',save_frequency,1000)
      call readi(mcmcard,'MESSAGE_FREQ',message_frequency,1000)
      call readi(mcmcard,'POOL_READ_FREQ',pool_read_freq,5000)
      call readi(mcmcard,'POOL_SAVE_FREQ',pool_save_freq,1000)
      call readi(mcmcard,'PRINT_FREQ',print_freq,1000)
      if (nwindow.gt.0) then
        read (inp,*) (winstart(i),winend(i),i=1,nwindow)
        do i=1,nwindow
          winlen(i)=winend(i)-winstart(i)+1
        enddo
      endif
      if (tmax.lt.tmin) tmax=tmin
      if (tmax.eq.tmin) then
        nstepc=0
        nsteph=0
      endif
      if (nstepc.gt.0 .and. nsteph.gt.0) then
        tsteph=(tmax/tmin)**(1.0D0/(nsteph+0.0D0)) 
        tstepc=(tmax/tmin)**(1.0D0/(nstepc+0.0D0)) 
      endif
C Probabilities of different move types
      sumpro_type(0)=0.0D0
      call reada(mcmcard,'MULTI_BOND',sumpro_type(1),1.0d0)
      call reada(mcmcard,'ONE_ANGLE' ,sumpro_type(2),2.0d0)
      sumpro_type(2)=sumpro_type(1)+sumpro_type(2)
      call reada(mcmcard,'THETA'     ,sumpro_type(3),0.0d0)
      sumpro_type(3)=sumpro_type(2)+sumpro_type(3)
      call reada(mcmcard,'SIDE_CHAIN',sumpro_type(4),0.5d0)
      sumpro_type(4)=sumpro_type(3)+sumpro_type(4)
      do i=1,MaxMoveType
        print *,'i',i,' sumprotype',sumpro_type(i)
        sumpro_type(i)=sumpro_type(i)/sumpro_type(MaxMoveType)
        print *,'i',i,' sumprotype',sumpro_type(i)
      enddo
      return
      end 
c----------------------------------------------------------------------------
      subroutine read_minim
      include 'DIMENSIONS'
      include 'COMMON.MINIM'
      include 'COMMON.IOUNITS'
      character*80 ucase
      character*320 minimcard
      call card_concat(minimcard)
      call readi(minimcard,'MAXMIN',maxmin,2000)
      call readi(minimcard,'MAXFUN',maxfun,5000)
      call reada(minimcard,'TOLF',tolf,1.0D-2)
      call reada(minimcard,'RTOLF',rtolf,1.0D-4)
      write (iout,'(/80(1h*)/20x,a/80(1h*))') 
     &         'Options in energy minimization:'
      write (iout,'(a,i5,a,i5,a,1pe14.5,a,1pe14.5)')
     & 'MaxMin:',MaxMin,' MaxFun:',MaxFun,' TolF:',TolF,' RTolF:',RTolF
      return
      end
c----------------------------------------------------------------------------
      subroutine read_angles(kanal,*)
      include 'DIMENSIONS'
      include 'COMMON.GEO'
      include 'COMMON.VAR'
      include 'COMMON.CHAIN'
      include 'COMMON.IOUNITS'
      read (kanal,*,err=10,end=10) (theta(i),i=3,nres)
      read (kanal,*,err=10,end=10) (phi(i),i=4,nres)
      read (kanal,*,err=10,end=10) (alph(i),i=2,nres-1)
      read (kanal,*,err=10,end=10) (omeg(i),i=2,nres-1)
      do i=1,nres
c 9/7/01 avoid 180 deg valence angle
        if (theta(i).gt.179.99d0) theta(i)=179.99d0
c
        theta(i)=deg2rad*theta(i)
        phi(i)=deg2rad*phi(i)
        alph(i)=deg2rad*alph(i)
        omeg(i)=deg2rad*omeg(i)
      enddo
      return
   10 return1
      end
c----------------------------------------------------------------------------
      subroutine reada(rekord,lancuch,wartosc,default)
      implicit none
      character*(*) rekord,lancuch
      double precision wartosc,default
      integer ilen,iread
      external ilen
      iread=index(rekord,lancuch)
      if (iread.eq.0) then
        wartosc=default 
        return
      endif   
      iread=iread+ilen(lancuch)+1
      read (rekord(iread:),*,err=10,end=10) wartosc
      return
  10  wartosc=default
      return
      end
c----------------------------------------------------------------------------
      subroutine readi(rekord,lancuch,wartosc,default)
      implicit none
      character*(*) rekord,lancuch
      integer wartosc,default
      integer ilen,iread
      external ilen
      iread=index(rekord,lancuch)
      if (iread.eq.0) then
        wartosc=default 
        return
      endif   
      iread=iread+ilen(lancuch)+1
      read (rekord(iread:),*,err=10,end=10) wartosc
      return
  10  wartosc=default
      return
      end
c----------------------------------------------------------------------------
      subroutine openunits
      include 'DIMENSIONS'    
#ifdef MP
      include 'mpif.h'
      include 'COMMON.INFO'
      include 'COMMON.SETUP'
      character*4 liczba
      character*16 form
      character*256 PREF_ORIG
#endif
      include 'COMMON.IOUNITS'
      integer lenpre,lenpot,ilen,lentmp
      external ilen
      call getenv_loc("PREFIX",prefix)
      call getenv_loc("POT",pot)
      call getenv_loc("TMPDIR",tmpdir)
      lenpre=ilen(prefix)
      lenpot=ilen(pot)
      lentmp=ilen(tmpdir)
      entname=prefix(:lenpre)//'_'//pot(:lenpot)//'.entr'
      PREF_ORIG = PREFIX
      if (lentmp.gt.0) then
      PREFIX = tmpdir(:lentmp)//'/'//prefix(:lenpre)
      lenpre = lentmp+lenpre+1
      endif
C Get the names and open the input files
#if defined(WINIFL) || defined(WINPGI)
      open(1,file=pref_orig(:ilen(pref_orig))//
     &  '.inp',status='old',readonly,shared)
C      open (9,file=prefix(:ilen(prefix))//'.intin',status='unknown')
C      open (18,file=prefix(:ilen(prefix))//'.entin',status='unknown')
C Get parameter filenames and open the parameter files.
      call getenv_loc('BONDPAR',bondname)
      open (ibond,file=bondname,status='old',readonly,shared)
      call getenv_loc('THETPAR',thetname)
      open (ithep,file=thetname,status='old',readonly,shared)
      call getenv_loc('ROTPAR',rotname)
      open (irotam,file=rotname,status='old',readonly,shared)
      call getenv_loc('TORPAR',torname)
      open (itorp,file=torname,status='old',readonly,shared)
      call getenv_loc('TORDPAR',tordname)
      open (itordp,file=tordname,status='old',readonly,shared)
      call getenv_loc('FOURIER',fouriername)
      open (ifourier,file=fouriername,status='old',readonly,shared)
      call getenv_loc('ELEPAR',elename)
      open (ielep,file=elename,status='old',readonly,shared)
      call getenv_loc('SIDEPAR',sidename)
      open (isidep,file=sidename,status='old',readonly,shared)
#elif (defined CRAY)
      open(1,file=pref_orig(:ilen(pref_orig))//'.inp',status='old',
     &  action='read')
C      open (9,file=prefix(:ilen(prefix))//'.intin',status='unknown')
C      open (18,file=prefix(:ilen(prefix))//'.entin',status='unknown')
C Get parameter filenames and open the parameter files.
      call getenv_loc('BONDPAR',bondname)
      open (ibond,file=bondname,status='old',action='read')
      call getenv_loc('THETPAR',thetname)
      open (ithep,file=thetname,status='old',action='read')
      call getenv_loc('ROTPAR',rotname)
      open (irotam,file=rotname,status='old',action='read')
      call getenv_loc('TORPAR',torname)
      open (itorp,file=torname,status='old',action='read')
      call getenv_loc('TORDPAR',tordname)
      open (itordp,file=tordname,status='old',action='read')
      call getenv_loc('FOURIER',fouriername)
      open (ifourier,file=fouriername,status='old',action='read')
      call getenv_loc('ELEPAR',elename)
      open (ielep,file=elename,status='old',action='read')
      call getenv_loc('SIDEPAR',sidename)
      open (isidep,file=sidename,status='old',action='read')
#else
      open(1,file=pref_orig(:ilen(pref_orig))//'.inp',status='old',
     &  readonly)
C      open (9,file=prefix(:ilen(prefix))//'.intin',status='unknown')
C      open (18,file=prefix(:ilen(prefix))//'.entin',status='unknown')
C Get parameter filenames and open the parameter files.
      call getenv_loc('BONDPAR',bondname)
      open (ibond,file=bondname,status='old',readonly)
      call getenv_loc('THETPAR',thetname)
      open (ithep,file=thetname,status='old',readonly)
      call getenv_loc('ROTPAR',rotname)
      open (irotam,file=rotname,status='old',readonly)
      call getenv_loc('TORPAR',torname)
      open (itorp,file=torname,status='old',readonly)
      call getenv_loc('TORDPAR',tordname)
      open (itordp,file=tordname,status='old',readonly)
      call getenv_loc('FOURIER',fouriername)
      open (ifourier,file=fouriername,status='old',readonly)
      call getenv_loc('ELEPAR',elename)
      open (ielep,file=elename,status='old',readonly)
      call getenv_loc('SIDEPAR',sidename)
      open (isidep,file=sidename,status='old',readonly)
#endif
#ifndef OLDSCP
C
C 8/9/01 In the newest version SCp interaction constants are read from a file
C Use -DOLDSCP to use hard-coded constants instead.
C
      call getenv_loc('SCPPAR',scpname)
#if defined(WINIFL) || defined(WINPGI)
      open (iscpp,file=scpname,status='old',readonly,shared)
#elif (defined CRAY)
      open (iscpp,file=scpname,status='old',action='read')
#else
      open (iscpp,file=scpname,status='old',readonly)
#endif
#endif
      call getenv_loc('PATTERN',patname)
#if defined(WINIFL) || defined(WINPGI)
      open (icbase,file=patname,status='old',readonly,shared)
#elif (defined CRAY)
      open (icbase,file=patname,status='old',action='read')
#else
      open (icbase,file=patname,status='old',readonly)
#endif
#ifdef MPL
      if (MyID.eq.BossID) then
      MyRank = MyID/fgProcs
#endif
#ifdef MP
      print *,'OpenUnits: processor',MyID,' MyRank=',MyRank
c      call numstr(MyRank,liczba)
      if (nprocs.eq.1) then
        npos=3
      else
        npos = dlog10(dfloat(nprocs-1))+1
      endif
      if (npos.lt.3) npos=3
      write (liczba,'(i1)') npos
      form = '(bz,i'//liczba(:ilen(liczba))//'.'//liczba(:ilen(liczba))
     &  //')'
      write (liczba,form) myrank
      outname=prefix(:lenpre)//'.out_'//pot(:lenpot)//
     &  liczba(:ilen(liczba))
      intname=prefix(:lenpre)//'_'//pot(:lenpot)//liczba(:ilen(liczba))
     &  //'.int'
      pdbname=prefix(:lenpre)//'_'//pot(:lenpot)//liczba(:ilen(liczba))
     &  //'.pdb'
      mol2name=prefix(:lenpre)//'_'//pot(:lenpot)//
     &  liczba(:ilen(liczba))//'.mol2'
      statname=prefix(:lenpre)//'_'//pot(:lenpot)//
     &  liczba(:ilen(liczba))//'.stat'
      rest2name=prefix(:lenpre)//"_"//liczba(:ilen(liczba))//'.rst' 
#else
      outname=prefix(:lenpre)//'.out_'//pot(:lenpot)
      intname=prefix(:lenpre)//'_'//pot(:lenpot)//'.int'
      pdbname=prefix(:lenpre)//'_'//pot(:lenpot)//'.pdb'
      mol2name=prefix(:lenpre)//'_'//pot(:lenpot)//'.mol2'
      statname=prefix(:lenpre)//'_'//pot(:lenpot)//'.stat'
      rest2name=prefix(:lenpre)//'.rst'
#endif
#if defined(AIX) || defined(PGI)
      open(iout,file=outname,status='unknown')
c      if(me.eq.king) then
c      open(igeom,file=intname,status='unknown',position='append')
c      open(ipdb,file=pdbname,status='unknown')
c      open(imol2,file=mol2name,status='unknown')
       open(istat,file=statname,status='unknown',position='append')
c      endif
#else
      open(iout,file=outname,status='unknown')
c      if(me.eq.king) then
c      open(igeom,file=intname,status='unknown',access='append')
c      open(ipdb,file=pdbname,status='unknown')
c      open(imol2,file=mol2name,status='unknown')
       open(istat,file=statname,status='unknown',access='append')
c      endif
#endif
#ifdef MPL
      endif ! (MyID.eq.BossID)
#endif
      csa_rbank=prefix(:lenpre)//'.CSA.rbank'
      csa_seed=prefix(:lenpre)//'.CSA.seed'
      csa_history=prefix(:lenpre)//'.CSA.history'
      csa_bank=prefix(:lenpre)//'.CSA.bank'
      csa_bank1=prefix(:lenpre)//'.CSA.bank1'
      csa_alpha=prefix(:lenpre)//'.CSA.alpha'
      csa_alpha1=prefix(:lenpre)//'.CSA.alpha1'
      csa_bankt=prefix(:lenpre)//'.CSA.bankt'
      csa_int=prefix(:lenpre)//'.int'
      csa_bank_reminimized=prefix(:lenpre)//'.CSA.bank_reminimized'
      csa_native_int=prefix(:lenpre)//'.CSA.native.int'
      csa_in=prefix(:lenpre)//'.CSA.in'
C Write file names
      write (iout,'(80(1h-))')
      write (iout,'(30x,a)') "FILE ASSIGNMENT"
      write (iout,'(80(1h-))')
      write (iout,*) "Input file                      : ",
     &  pref_orig(:ilen(pref_orig))//'.inp'
      write (iout,*) "Output file                     : ",
     &  outname(:ilen(outname))
      write (iout,*)
      write (iout,*) "Sidechain potential file        : ",
     &  sidename(:ilen(sidename))
#ifndef OLDSCP
      write (iout,*) "SCp potential file              : ",
     &  scpname(:ilen(scpname))
#endif
      write (iout,*) "Electrostatic potential file    : ",
     &  elename(:ilen(elename))
      write (iout,*) "Cumulant coefficient file       : ",
     &  fouriername(:ilen(fouriername))
      write (iout,*) "Torsional parameter file        : ",
     &  torname(:ilen(torname))
      write (iout,*) "Double torsional parameter file : ",
     &  tordname(:ilen(tordname))
      write (iout,*) "Bond & inertia constant file    : ",
     &  bondname(:ilen(bondname))
      write (iout,*) "Bending parameter file          : ",
     &  thetname(:ilen(thetname))
      write (iout,*) "Rotamer parameter file          : ",
     &  rotname(:ilen(rotname))
      write (iout,*) "Threading database              : ",
     &  patname(:ilen(patname))
      if (lentmp.ne.0) 
     &write (iout,*)" DIRTMP                          : ",
     &  tmpdir(:lentmp)
      write (iout,'(80(1h-))')
      return
      end
c----------------------------------------------------------------------------
      subroutine card_concat(card)
      include 'DIMENSIONS'
      include 'COMMON.IOUNITS'
      character*(*) card
      character*80 karta,ucase
      external ilen
      read (inp,'(a)') karta
      karta=ucase(karta)
      card=' '
      do while (karta(80:80).eq.'&')
        card=card(:ilen(card)+1)//karta(:79)
        read (inp,'(a)') karta
        karta=ucase(karta)
      enddo
      card=card(:ilen(card)+1)//karta
      return
      end
c----------------------------------------------------------------------------------
      subroutine readrst
      include 'DIMENSIONS'
      include 'COMMON.CHAIN'
      include 'COMMON.IOUNITS'
      include 'COMMON.MD'
      open(irest2,file=rest2name,status='unknown')
      read(irest2,*) totT,EK,potE,totE
      do i=1,2*nres
         read(irest2,'(3e15.5)') (d_t(j,i),j=1,3)
      enddo
      do i=1,2*nres
         read(irest2,'(3e15.5)') (dc(j,i),j=1,3)
      enddo
      close(irest2)
      return
      end
c---------------------------------------------------------------------------------
#ifdef WINIFL
      subroutine flush(iu)
      return
      end
#endif
